/**
 * The <code>PINAClient</code> class
 *
 * @author Jianmin Wu
 * @version 1.01 July 10, 2011
 */

package edu.helsinki.biomedicum.interactome.ws_rest;

import com.sun.jersey.api.client.Client;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.WebResource;
import com.sun.jersey.api.client.config.ClientConfig;
import com.sun.jersey.api.client.config.DefaultClientConfig;
import com.sun.jersey.api.representation.Form;

import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.UriBuilder;
import java.io.*;
import java.util.LinkedList;
import java.util.List;

public class PINAClient {
    public static final String WS_URL = "http://cbg.garvan.unsw.edu.au/pina/restWS/current/";
    public static final int TIME_OUT = 2000000; // milli seconds
    public static final int MAX_NUMBER_ONCE_QUERY_INTERACTORS = 500;
    public static final int MAX_NUMBER_ONCE_QUERY_INTERACTIONS = 3000;


    public static void main(String args[]) {
        try {
            if (args.length != 4) {
                System.err.println("Please set values for all properties in the file build.properties.");
                System.exit(-1);
            }

            File inputFile = new File(args[0]);
            if (!inputFile.exists()) {
                System.err.println("The input file " + inputFile + " doens't exist.");
                System.exit(-1);
            }
            File outputFile = new File(args[1]);

            String format = args[2];
            boolean queryInteractors = Boolean.valueOf(args[3]);


            PINAClient client = new PINAClient();
            System.out.println("Reading input from file:" + inputFile.getAbsolutePath());
            List<String> list = client.readInputFile(inputFile);


            ClientConfig config = new DefaultClientConfig();
            Client stub = Client.create(config);
            WebResource service = stub.resource(UriBuilder.fromUri(WS_URL).build());
            // Fluent interfaces

            String output = "";
            if (queryInteractors) {
                System.out.println("Query for interactors:");
                output = client.queryInteractors(service, format, list);
            } else {
                System.out.println("Query whether interactions are known:");
                output = client.queryInteractions(service, format, list);
            }

            System.out.println("Writing result to file:" + outputFile.getAbsolutePath());
            FileWriter fw = new FileWriter(outputFile);
            BufferedWriter bw = new BufferedWriter(fw);
            PrintWriter pw = new PrintWriter(bw);
            pw.println(output);
            pw.flush();
            pw.close();

        } catch (Exception e) {
            e.printStackTrace();
            System.exit(-1);
        }
    }


    public List<String> readInputFile(File f) {
        List<String> inputList = new LinkedList<String>();
        String line = null;
        try {
            FileReader fr = new FileReader(f);
            BufferedReader br = new BufferedReader(fr);
            while (null != (line = br.readLine())) {
                line = line.trim();
                if (line.length() > 0 && !line.startsWith("#"))
                    inputList.add(line);
            }
        } catch (IOException e) {
            e.printStackTrace();
            System.exit(-1);
        }
        return inputList;
    }

    /**
     * Example for query interacting proteins of a list of Uniprot AC
     *
     * @param service
     * @param format  specify the format of the result. Two options: plain or graphML
     * @param list
     * @return
     * @throws java.io.IOException
     */
    public String queryInteractors(WebResource service, String format, List<String> list) throws IOException {
        if (list.size() <= MAX_NUMBER_ONCE_QUERY_INTERACTORS)
            return subQueryInteractors(service, list, format);
        else {
            StringBuffer sb = new StringBuffer();
            for (int i = 0; i < list.size() / MAX_NUMBER_ONCE_QUERY_INTERACTORS; i++) {
                if (i + 1 <= list.size() / MAX_NUMBER_ONCE_QUERY_INTERACTORS)
                    sb.append(subQueryInteractors(service, list.subList(MAX_NUMBER_ONCE_QUERY_INTERACTORS * i, MAX_NUMBER_ONCE_QUERY_INTERACTORS * (i + 1)), format));
                else
                    sb.append(subQueryInteractors(service, list.subList(MAX_NUMBER_ONCE_QUERY_INTERACTORS * i, list.size()), format));
            }
            return sb.toString();
        }
    }

    private String subQueryInteractors(WebResource service, List<String> list, String format) throws IOException {
        StringBuffer queryProteins = new StringBuffer(list.size());

        for (int i = 0; i < list.size(); i++) {
            queryProteins.append(list.get(i)).append(";");
        }

        Form form = new Form();
        form.add("ids", queryProteins.toString());

        if (format.equalsIgnoreCase("xml")) {
            ClientResponse response = service.path("getInteractors").type(MediaType.APPLICATION_FORM_URLENCODED).accept(MediaType.TEXT_XML)
                    .post(ClientResponse.class, form);

            return response.getEntity(String.class);
        } else {
            ClientResponse response = service.path("getInteractors").type(MediaType.APPLICATION_FORM_URLENCODED).accept(MediaType.TEXT_PLAIN)
                    .post(ClientResponse.class, form);

            return response.getEntity(String.class);
        }
    }


    /**
     * Example for querying a list of interactions whether they exist
     *
     * @param service
     * @param format  specify the format of the result. Two options: plain or graphML
     * @param list
     * @return
     * @throws java.io.IOException
     */
    public String queryInteractions(WebResource service, String format, List<String> list) throws IOException {
        if (list.size() <= MAX_NUMBER_ONCE_QUERY_INTERACTIONS)
            return subQueryInteractions(service, list, format);
        else {
            StringBuffer sb = new StringBuffer();
            for (int i = 0; i < list.size() / MAX_NUMBER_ONCE_QUERY_INTERACTIONS; i++) {
                if (i + 1 <= list.size() / MAX_NUMBER_ONCE_QUERY_INTERACTIONS)
                    sb.append(subQueryInteractions(service, list.subList(MAX_NUMBER_ONCE_QUERY_INTERACTIONS * i, MAX_NUMBER_ONCE_QUERY_INTERACTIONS * (i + 1)), format));
                else
                    sb.append(subQueryInteractions(service, list.subList(MAX_NUMBER_ONCE_QUERY_INTERACTIONS * i, list.size()), format));
            }
            return sb.toString();
        }

    }

    private String subQueryInteractions(WebResource service, List<String> list, String format) throws IOException {
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < list.size(); i++) {
            String line = list.get(i).trim();
            sb.append(line).append("\n");
        }

        Form form = new Form();
        form.add("pairs", sb.toString());

        if (format.equalsIgnoreCase("xml")) {
            ClientResponse response = service.path("getInteractions").type(MediaType.APPLICATION_FORM_URLENCODED).accept(MediaType.TEXT_XML)
                    .post(ClientResponse.class, form);

            return response.getEntity(String.class);
        } else {
            ClientResponse response = service.path("getInteractions").type(MediaType.APPLICATION_FORM_URLENCODED).accept(MediaType.TEXT_PLAIN)
                    .post(ClientResponse.class, form);
            return response.getEntity(String.class);
        }
    }


}
